%
%
% Used for output for paper
%
%
%
% Mass health plan choice data
% focus only on cells with
% choices 1,2,4,5 (code 27)

tic
rng(937162211);

nbsim = 5000;           % set number of bootstrap iterations
alpha = .05;            % set size

Tab = readtable('pp_data_v2_copy.csv');     % import data
mtab = table2array(Tab);

im = (mtab(:,7)==27);
imtab = mtab(im,:);     % extract cells with choice set code 27 (1,2,4,5)
mtab = imtab;

clear imtab im

mmtab = mtab;
jjc = isnan(mmtab(:,11));   % delete choices with price=nan (unavailable choices)
mmtab(jjc,:) = [];
jjc = isnan(mmtab(:,12));   % delete choices with price=nan (unavailable choices)
mmtab(jjc,:) = [];
jjc = (mmtab(:,9)>3);       % re-number choices 4 & 5 to 3 & 4 (-> choices now 1,2,3,4)
mmtab(jjc,9) = mmtab(jjc,9)-1;
jjc = (mmtab(:,10)>3);
mmtab(jjc,10) = mmtab(jjc,10)-1;
nnc = size(mmtab,1)/16;     % number of cells
n27 = sum(mmtab(:,13));     % total # consumers

% data structure
% 1120 rows:
% 70 cells,
% 4 choices in each year = 16 pairs of choices,
% 70 x 16 = 1120 rows

% 13 columns:
% col 1: cell #
% col 2&3: year pair
% col 4: region
% col 5&6: income each year
% col 7: choice set code
% col 8: cell size (#consumers in cell)
% col 9&10: plan in each year (1-4)
% col 11&12: prices in each year
% col 13: # consumers choosing this choice pair in this cell


nnvec = mmtab(:,13);    % # consumers choosing choice pair by cell

bcum = cumsum(nnvec/n27);     % cumulative fraction nnvec
bcum(end)=1;


kn = sqrt(.3*log(n27));         % kn and bn are Andrews-Shi constants for
% slackness (see section 9 of Andrews-Shi)
bn = sqrt( (.4*log(n27))/(log(log(n27))) );


% generate bootstrap samples which will be used for each theta
bnvecm = [];    % each column is bootstrap sample
% with n27 (13169) observations generated as multinomial
% draw with probabilities for each cell choice pair equal
% to the empirical probabilities

for iibb = 1:nbsim
    bu = rand(1,n27);
    ib = (bu < bcum);
    vib = nnc*16+1-sum(ib);

    clear ib

    bnvec = [];

    for jjb = 1:(nnc*16)
        bnvec = [bnvec; sum(vib == jjb)];
    end
    bnvecm = [bnvecm bnvec];
end



nr = 4;
nnr = 2;
vnr = 1:4;
vnr = vnr';

%  create mom data starter kit
momdd = [];     % 16 columns for each choice pair
% 14 rows  = (4 choose 1) + (4 choose 2) + (4 choose 3)
%

for ys = 1:4
    for yt = 1:4
        mvec=[];

        for ii = 1:2              % iterate through choice set sizes
            cv = nchoosek(vnr,ii);   % all combos of length ii in vnr
            ncv = size(cv,1);
            for jj = 1:ncv          % iterate thru choice sets of size ii
                cvjj = cv(jj,:);
                cvjj = cvjj';
                cvcjj = vnr;
                cvcjj(cvjj) = [];

                if ~isempty(intersect(cvjj,ys)) & isempty(intersect(cvjj,yt))
                    mvec = [mvec; 1];

                elseif isempty(intersect(cvjj,ys)) & ~isempty(intersect(cvjj,yt))
                    mvec = [mvec; -1];
                else
                    mvec = [mvec; 0];
                end

                if ii==1            % choice sets of size 3 (as complement of choice sets of size 1)
                    if ~isempty(intersect(cvcjj,ys)) & isempty(intersect(cvcjj,yt))
                        mvec = [mvec; 1];

                    elseif isempty(intersect(cvcjj,ys)) & ~isempty(intersect(cvcjj,yt))
                        mvec = [mvec; -1];
                    else
                        mvec = [mvec; 0];
                    end
                end
            end
        end
        momdd = [momdd mvec];

    end
end




critneg = [];
critmax = [];
bcvneg = [];
bcvmax = [];

%%%%%% iterate over grid of theta values
%  grid on [-.85,1] at 1/1000 grid increments
for itheta = 0:1850

    if itheta == round(itheta/10)*10
        itheta
    end
    cmom1a = [];    % year pair a (2010, 2012), b (2012, 2013)
    cmom1b = [];
    miv1 = [];



    theta = [1;((itheta/1000)-.85)];

    for ic = 1:nnc      % iterate over cells

        imtab = mmtab((ic*16-15):(ic*16),:);    % data for cell ic

        dx = [];        % vector of covar index diffs for each choice pair in cell ic
        mdxi = [];

        for iiv = 1:nr          % iterate over choice pairs
            jjv = (imtab(:,9)==iiv &   imtab(:,10)==iiv);   % indicator for row containing choice pair


            % price coeff theta(1) or theta(1)+theta(2) for pd s and t
            % ths and tht are covar index for choice pair jjv
            ths = theta(1) + (imtab(jjv,5) > 3.5)*theta(2);
            tht = theta(1) + (imtab(jjv,6) > 3.5)*theta(2);
            dx = [dx; -imtab(jjv,11).*ths + imtab(jjv,12).*tht];     %covar index diff

        end




        for ii = 1:nnr              % iterate through choice set sizes
            cv = nchoosek(vnr,ii);  % all choice subsets of size ii
            ncv = size(cv,1);
            for jj = 1:ncv          % iterate thru choice subsets of size ii
                cvjj = cv(jj,:);
                cvjj = cvjj';
                cvcjj = vnr;
                cvcjj(cvjj) = [];   % choice subset complement

                dx1 = dx(cvjj);     % covar index diffs for choice subset
                dx2 = dx;
                dx2(cvjj) = [];     % covar index diffs for complement of choice subset


                if min(dx1) >= max(dx2)     % tracking which choice subsets are "maximal"
                    mdxi = [mdxi; 1];

                else
                    mdxi = [mdxi; 0];
                end

                % When subset size ii=1, then take care of subset size ii=3
                if ii ~= (nr/2)
                    if min(dx2) >= max(dx1)
                        mdxi = [mdxi; 1];
                    else
                        mdxi = [mdxi; 0];
                    end

                end
            end
        end
        mdx = mdxi*ones(1,16);
        momddi = momdd.*mdx;


        regyr = mmtab(ic*16,2);
        ric1a = 0;
        ric1b = 0;
        if (regyr ==2010)
            ric1a = 1;
        else
            ric1b = 1;

        end

        cmom1a = [cmom1a ric1a*momddi];
        cmom1b = [cmom1b ric1b*momddi];


        miv1 = [miv1 momddi];
    end

    cmom = [cmom1a; cmom1b];


    % turn cmom into average of uncondl moms where instrument is cell
    cmbar = bsxfun(@times,nnvec',cmom);
    cmbar = sum(cmbar')/n27;
    cmbar = cmbar';

    % compute no instrument sample moment
    cmm = bsxfun(@times,nnvec',miv1);
    cmm = sum(cmm')/n27;
    cmm = cmm';


    % compute variance

    sigm = bsxfun(@minus,cmom,cmbar);
    sigm = sigm.*sigm;
    sigm = bsxfun(@times,nnvec',sigm);
    sigm = sum(sigm')/n27;
    sigm = sigm';

    iism = (sigm==0);
    sigm(iism) = sigm(iism) + .01;     % avoid division by zero (no effect on results)



    % compute no instrument variance for normalization in simulation
    % (following Andrews-Shi)

    sigd = bsxfun(@minus,miv1,cmm);
    sigd = sigd.*sigd;
    sigd = bsxfun(@times,nnvec',sigd);
    sigd = sum(sigd')/n27;
    sigd = sigd';

    iisd = (sigd==0);
    sigd(iisd) = sigd(iisd) +  .01;   % avoid division by zero (no effect on results)



    vsd = kron(ones(2,1),(1./sqrt(sigd)) );



    % compute criterion function values

    cmbars = sqrt(n27)*cmbar./sqrt(sigm);

    iis1 = (cmbars<0);
    if sum(iis1)>0
        critneg = [critneg ; theta(2) sum(cmbars(iis1).*cmbars(iis1))];
        critmax = [critmax ; theta(2) max(cmbars(iis1).*cmbars(iis1))];
    else
        critneg = [critneg ; theta(2) 0];
        critmax = [critmax ; theta(2) 0];
    end





    % bootstrap

    % recentering bootstrap vector
    bxi = cmbars/kn;
    bpsi = bn*(bxi > 1);

    ibcritneg = [];
    ibcritmax= [];

    for iibb = 1:nbsim

        bnvec = bnvecm(:,iibb);

        bcmbar = bsxfun(@times,bnvec',cmom);
        bcmbar = sum(bcmbar')/n27;
        bcmbar = bcmbar';

        bbcmbar = sqrt(n27)*(vsd.*(bcmbar-cmbar));
        bbcmbar = bbcmbar + bpsi;


        % compute bootstrap variance

        bsigm = bsxfun(@minus,cmom,bcmbar);
        bsigm = bsigm.*bsigm;
        bsigm = bsxfun(@times,bnvec',bsigm);
        bsigm = sum(bsigm')/n27;
        bsigm = bsigm';
        bsigm = bsigm.*vsd.*vsd;

        iibsm = (bsigm==0);
        bsigm(iibsm) = bsigm(iibsm) +  .01;  % avoid division by zero (no effect on results)




        % compute criterion function values

        bbcmbars = bbcmbar./sqrt(bsigm);

        iisbb = (bbcmbars<0);
        if sum(iisbb)>0
            ibcritneg = [ibcritneg ; sum(bbcmbars(iisbb).*bbcmbars(iisbb))];
            ibcritmax = [ibcritmax ; max(bbcmbars(iisbb).*bbcmbars(iisbb))];
        else
            ibcritneg = [ibcritneg ; 0];
            ibcritmax = [ibcritmax ; 0];
        end

    end

    bcvneg = [bcvneg; theta(2) quantile(ibcritneg, 1-alpha)];
    bcvmax = [bcvmax; theta(2) quantile(ibcritmax, 1-alpha)];




end


% estimated confidence set values
aa = (critneg(:,2) <= bcvneg(:,2));
minci = min(critneg(aa,1));
maxci = max(critneg(aa,1));
fprintf('CI is [ %2.4f ,  %2.4f ] \n\n',minci,maxci)

% check that CI is connected:  true if to values below are equal
fprintf('If following values are equal, CI is connected\n')
[((maxci-minci)*1000+1)   length(critneg(aa,1))]

% graph of criterion function and bootstrap critical value function
plot(critneg(:,1),critneg(:,2),bcvneg(:,1),bcvneg(:,2),'LineWidth',1.8)
xlabel('$\gamma_0$','interpreter','latex')
ylim([0,130])
xlim([-1,1.1])

toc
